; Includes
#include <reg420.h>
#include <SPIMacros.inc>

; Local Definitions

STACK           equ  080h	  ; Stack Location
OUTPUT_STRING   equ  0300h	  ; Location of String
RELOAD_VALUE    equ  0FDh     ; 57600 @ 33MHz

;****************************************************************************
; Application
;****************************************************************************

ORG 0000h			  ; Reset Vector
    ajmp  cfLCD

org 0100h

M_WELCOME:
DB "DS89C420 Software SPI Application Note",13,10,0

cfLCD:
    mov   sp,     #STACK    ; Initialize stack pointer so we can call functions
    acall cfLCD_Init
    mov   a, #0Ch           ; Clear Screen Char
    PUTX
    inc   dptr
    mov   a, #1             ; Home Position Char
    PUTX
    mov   dptr, #OUTPUT_STRING
    mov   r0, #2
    acall xmitSPI           ; Initialize the LCD
cfLCD_TOP:
    mov   a, #0Ch           ; Clear Screen Char
    PUTX
    inc   dptr
    mov   a, #1             ; Home Position Char
    PUTX
    inc   dptr
    mov   r0, #2
    mov   a, #0Dh
    acall SendByte
    mov   a, #0Ah
    acall SendByte
    mov   a, #'='
    acall SendByte
    mov   a, #'>'
    acall SendByte          
    mov   a, #20h
    acall SendByte          ; Output our prompt
cfLCD_Reading:
    acall ReceiveByte       ; Get the new byte
    acall SendByte          ; Echo it back
    PUTX
    inc   dptr
    inc   r0                ; Add another byte to our length
    cjne  r0,   #0FFh, cfLCD_NotMaxLength
    sjmp  cfLCD_SendSPI
cfLCD_NotMaxLength:
    cjne  a,    #0Dh, cfLCD_Reading
cfLCD_SendSPI:
    mov   a,    #0Dh
    acall SendByte
    mov   a,    #0Ah
    acall SendByte
    mov   dptr, #OUTPUT_STRING
    acall xmitSPI           ; Write the data we received over SPI
    sjmp  cfLCD_TOP

;****************************************************************************

SendString:
    push    acc
SendStringLoop:
    clr     A
    movc    A,@A+dptr
    jz      SendStringEnd
    acall   SendByte
    inc     dptr
    sjmp    SendStringLoop
SendStringEnd:
    pop     acc
    ret

;****************************************************************************

ReceiveByte:
    jnb     RI,$
    mov     a,  SBUF0
    clr     RI
    ret

;****************************************************************************

SendByte:
    clr     TI
    mov     SBuf0,A
    jnb     TI,$
    clr     TI
    ret

;****************************************************************************

cfLCD_Init:
    mov   psw,    #0        ; Put in correct register bank
    orl   PMR,    #01h      ; Initialize the internal data memory to accessible
    mov   r4,     #55       ; Set Clock delay to 55 us
    clr   a                 ; Initalize options to none
    setb  CKPOL             ; Set CKPOL true
    setb  CKPHA             ; Set CKPHA true
    setb  SKEW              ; Set No_Skew true
    setb  USE_SS            ; Set Use_SS true
    setb  SYNCH_SS          ; Bring SS inactive after every byte
    mov   r5,     a         ; Load Options into option register
; Leave SS set to active low, 8bit mode, and do synch words
    mov   r0,     #0
    mov   r1,     #0
    mov   r2,     #0
    mov   r3,     #0       
cfLCD_WarmUp:
    djnz  r1, cfLCD_WarmUp
    djnz  r2, cfLCD_WarmUp  ; Allow oscillator and LCD to warm up
; Set up Serial Port 0
    clr   TR1               ; Disable timer1
    mov   T2CON, #00h       ; Disable timer2
    mov   SCON0, #050h      ; Put Serial 0 into mode 1 and enable Receive
    mov   TMOD,  #020h      ; Put Timer1 into 8 bit auto reload mode
    anl   CKMOD, #0EFh      ; Clear T1MH
    anl   CKCON, #0EFh      ; Clear T1M set for clock divide by 12
    orl   PCON,  #080h      ; SET to double baud rate
    mov   TH1,   #RELOAD_VALUE       ; Load the count reset value
    mov   TL1,   #RELOAD_VALUE       ; Load the count reset value
    mov   T2CON, #00h
    setb  TR1
; Output welcome message to user on Serial Terminal
    mov   dps,    #0
    mov   dptr,   #M_WELCOME
    acall SendString
    mov   dptr,   #OUTPUT_STRING    ; Load data pointer to point to the data
    ret

;****************************************************************************
;
; SPI library
;
;****************************************************************************

;***************************************************************************
;*
;* Function Name: xmitSPI
;*
;*   Description: Write to/read from the makeshift (bit-bang) SPI port.
;*
;*      Input(s): R3:R0 - Length of transfer
;*              : R4    - Delay;     value*1us delay spreads clock
;*              : R5    - Options;  SPI configuration options defined as:
;* 	                bit 0 - CPOL - Set to 1 - SPICLK idles high
;*                  bit 1 - CPHA - Set to 1 - Transfers on second edge 
;*                  bit 2 - wordMode - Set to 1 - 16 bit transfers
;*	                bit 3 -	noskew - Set to 1 - turn off interrupts during transfer 	
;*  	            bit 4 - useSS - Set to 1 - Use the SS line during transfers
;*                  bit 5 - synchSS - Set to 1 - Takes SS to inactive after every word
;* 		            bit 6 - invertSS - Set to 1 - SS line is active high
;*                  bit 7 - undefined
;*
;*    Outputs(s): A - 0 if success, exception code otherwise
;*              
;****************************************************************************
xmitSPI:
	using  0
    push   dpl
    push   dph                      ; save the data pointer
    mov    a, r0                    ; We know r3 must be 0, test r2:r1:r0.
    orl    a, r1
    orl    a, r2
    jnz    initialize_clock         ; Test for 0 length read.
    ajmp   ok

initialize_clock:                   ; Start the clock in the correct state
    mov	   a, r5
	jnb    CKPOL, init_clk_low
    SET_CLK
    sjmp   initialize_SS
init_clk_low:
	CLEAR_CLK

initialize_SS:		                ; Start SS in the correct state
	mov    a, r5
	jnb    USE_SS, adjust_counter
	jnb    INVERT_SS, normal_ss
	anl    p2, #NOT_SS_MASK   ; Initialize SS to low
	sjmp   adjust_counter
normal_ss:
	orl    p2, #SS_MASK	    ; Initialize SS to high

adjust_counter:
    acall  spi_AdjustForDJNZ        ; Adjust r3-r0 for use as djnz counter.
    mov    a, r5 
    push   ie
	jnb    SKEW, set_delay_flag     ; Test no_skew boolean.
    clr    ea                       ; Disable interrupts.

check_wordSize:
	mov    R2_B1, #1
	mov    a, r5
	jnb    WORDMODE, set_delay_flag
	inc    R2_B1		            ; Set 16 bit transfer mode

;
; Note: We'll use the carry as a quick delay flag. This means that the
; entire loop must avoid using the carry or preserve it.
;
set_delay_flag:
    clr    c
    mov    a, r4
    jz     sync_ss_start            ; Test for 0 delay.
    setb   c

sync_ss_start:	; If choosing not to synch SS, activate SS here.
	mov    a, r5
	jnb    USE_SS,   handle_modes
	jb     SYNCH_SS, handle_modes
	jb     INVERT_SS, sync_ss_start_invert
	anl    p2, #NOT_SS_MASK
	sjmp   handle_modes
sync_ss_start_invert:
	orl    p2, #SS_MASK

;
; Check here to handle different SPI Modes CKPOL and CKPHA
;
handle_modes:
    mov     a, r5                   ; Check CKPHA
    jnb     CKPHA, inphase
	mov     a, r5
    jnb     CKPOL, normal_polarity   ; Check CKPOL
    acall   SPI_ModeD
    sjmp    sync_ss_stop

normal_polarity:
    acall   SPI_ModeC
    sjmp    sync_ss_stop
 
inphase:
    mov     a, r5
    jnb     CKPOL, normal_polarity_2 ; Check CKPOL
    acall   SPI_ModeB
    sjmp    sync_ss_stop

normal_polarity_2:
    acall   SPI_ModeA


sync_ss_stop:	   ; If choosing not to synch SS, deactivate SS here.
	mov     a, r5
	jnb     USE_SS,   transfer_finished
	jb      SYNCH_SS, transfer_finished
	jb      INVERT_SS, sync_ss_stop_invert
	orl     p2, #SS_MASK
	sjmp    transfer_finished
sync_ss_stop_invert:
	anl     p2, #NOT_SS_MASK
;
; We're finished with the write operation, restore interrupts
; and return.
;
transfer_finished:
    pop     ie                    ; Restore interrupt state.

ok:
    pop     dph
    pop     dpl                   ; Restore Data Pointer
    clr     a
    ret

;****************************************************************************
;*
;* Function Name: SPI_ModeA
;*
;*   Description: This subroutine handles Mode A SPI.  This corresponds to
;*                CKPOL = 0 and CKPHA = 0
;*
;*      Input(s):
;*
;*    Outputs(s):
;*
;*         Notes:
;*
;****************************************************************************
SPI_ModeA:
	mov     r6, R2_B1            ; Initialize the transfer size counter
    CLEAR_SS                     ; Clear SS line to active state if required
    jnc     modea_no_ss
    SPI_DELAY           
    SPI_DELAY 	                 ; Delay for a full clock cycle

modea_no_ss:
    mov     r7, #8               ; Initialize loop counter.
    mov     R1_B1, #0            ; Clear destination register.
    GETX                         ; Fetch byte from source buffer.
    mov     R0_B1, a

bang_bits_modea:
    CLEAR_CLK
	SET_DATA
    jnc     no_delay_modea       ; Test for delay.
    SPI_DELAY	                 ; Delay for ~.5us * delay here

no_delay_modea:                  ; Correction for sclk duty cycle
    mov     R4_B1, #3
duty_balance_loop_modea:
    djnz    R4_B1, duty_balance_loop_modea

after_delay_modea:
    SET_CLK
    GET_DATA
    djnz    r7, data_rotate_a    ; Check for bit loop termination
	nop
	nop
	nop
	nop	; More spi clock balancing and slowing....
	nop	; 10 Cycles to equal other path
	nop
	nop
	nop
	nop
	nop
    jnc     no_delay1_modea
    SPI_DELAY
no_delay1_modea:
    sjmp    put_read_val_modea
data_rotate_a:
	mov    a, R1_B1              ; Rotate destination reg
	rl    a
	mov   R1_B1, a       
	mov   a, R0_B1	             ; Rotate source reg 
	rl    a
	mov   R0_B1, a
    jnc   no_delay_end_modea
    SPI_DELAY	                 ; Delay for ~.5us * delay here
no_delay_end_modea:
	ajmp  bang_bits_modea

put_read_val_modea:              ; Make sure that SPI_CLK line returns
    CLEAR_CLK                    ; to inactive after byte transfer
    mov     a, R1_B1
    PUTX                         ; Store byte in destination buffer.
    inc     dptr                 ; Advance buffer pointer.
	djnz    r6, modea_no_ss
loop_ss_modea:
    SET_SS                       ; Set SS back to inactive state, to frame data, if required
    jnc     data_loop_modea
    SPI_DELAY                    ; If using delay, make sure SS stays inactive for sufficent time
	SPI_DELAY  
data_loop_modea:
    djnz    r0, modea_not_done
    djnz    r1, modea_not_done  ; Test for byte loop termination.
    djnz    r2, modea_not_done
    djnz    r3, modea_not_done
    ret
modea_not_done:
	ajmp    SPI_ModeA

;****************************************************************************
;*
;* Function Name: SPI_ModeB
;*
;*   Description: This subroutine handles Mode B SPI.  This corresponds to
;*                CKPOL = 1 and CKPHA = 0
;*
;*      Input(s):
;*
;*    Outputs(s):
;*
;*         Notes:
;*
;****************************************************************************
SPI_ModeB:
    mov     r6, R2_B1            ; Initialize the transfer size counter
    CLEAR_SS                     ; Clear SS line to active state if required
    jnc     modeb_no_ss
    SPI_DELAY
    SPI_DELAY		             ; Delay for a full clock cycle

modeb_no_ss:
    mov     r7, #8               ; Initialize loop counter.
    mov     R1_B1, #0            ; Clear destination register.
    GETX                         ; Fetch byte from source buffer.
    mov     R0_B1, a

bang_bits_modeb:
    SET_CLK
    SET_DATA
    jnc     no_delay_modeb    ; Test for delay.
    SPI_DELAY	              ; Delay for ~.5us * delay here

no_delay_modeb:               ; Correction for sclk duty cycle
    mov     R4_B1, #3
duty_balance_loop_modeb:
    djnz    R4_B1, duty_balance_loop_modeb

after_delay_modeb:
    CLEAR_CLK
    GET_DATA
    djnz  r7, data_rotate_b       ; Check for bit loop termination
	nop
	nop
	nop
	nop	; More spi clock balancing and slowing....
	nop	; 10 Cycles to equal other path
	nop
	nop
	nop
	nop
	nop
	jnc   no_delay1_modeb
	SPI_DELAY
no_delay1_modeb:
    sjmp  put_read_val_modeb
data_rotate_b:
	mov   a, R1_B1               ; Rotate destination reg
	rl    a
	mov   R1_B1, a       
	mov   a, R0_B1	             ; Rotate source reg 
	rl    a
	mov   R0_B1, a
    jnc   no_delay_end_modeb
    SPI_DELAY	                 ; Delay for ~.5us * delay here
no_delay_end_modeb:
	ajmp  bang_bits_modeb

put_read_val_modeb:              ; Make sure that SPI_CLK line returns to
    SET_CLK                      ; inactive after byte transfer
    mov      a, R1_B1
    PUTX                         ; Store byte in destination buffer.
    inc      dptr                   ; Advance buffer pointer.
	djnz     r6, modeb_no_ss
loop_ss_modeb:
    SET_SS                       ; Set SS back to inactive state, to frame data, if required
    jnc      data_loop_modeb
    SPI_DELAY                    ; If using delay, make sure SS stays inactive for sufficent time
	SPI_DELAY
data_loop_modeb:
    djnz    r0, modeb_not_done
    djnz    r1, modeb_not_done   ; Test for byte loop termination.
    djnz    r2, modeb_not_done
    djnz    r3, modeb_not_done
    ret
modeb_not_done:
	ajmp    SPI_ModeB

;****************************************************************************
;*
;* Function Name: SPI_ModeC
;*
;*   Description: This subroutine handles Mode C SPI.  This corresponds to
;*                CKPOL = 0 and CKPHA = 1
;*
;*      Input(s):
;*
;*    Outputs(s):
;*
;*         Notes:
;*
;****************************************************************************
SPI_ModeC:
    mov   r6, R2_B1		         ; Initialize the transfer size counter
    CLEAR_SS                     ; Clear SS line to active state if required
    jnc   modec_no_ss
    SPI_DELAY
    SPI_DELAY			         ; Delay for a full clock cycle

modec_no_ss:
    GETX                         ; Fetch byte from source buffer.
    mov   R0_B1, a
    mov   r7, #8                 ; Initialize loop counter.
    mov   R1_B1, #0              ; Clear destination register.

bang_bits_modec:
    SET_CLK
    SET_DATA
    jnc   no_delay_modec    ; Test for delay.
    SPI_DELAY	        	; Delay for ~.5us * delay here

no_delay_modec:             ; Correct for 50% duty here
    mov   R4_B1, #2
duty_balance_loop_modec:
    nop
    nop
    djnz  R4_B1, duty_balance_loop_modec

after_delay_modec:
    CLEAR_CLK
    GET_DATA
    djnz  r7, data_rotate_c ; Check for bit loop termination
    nop
    nop
    nop
    nop		; More spi clock balancing and slowing....
    nop		; 10 Cycles to equal other path
    nop
    nop
    nop
    nop
    nop
    jnc   no_delay1_modec
    SPI_DELAY
no_delay1_modec:
    sjmp  put_read_val_modec
data_rotate_c:
    mov   a, R1_B1       ; Rotate destination reg
    rl    a
    mov   R1_B1, a       
    mov   a, R0_B1	     ; Rotate source reg 
    rl    a
    mov   R0_B1, a
    jnc   no_delay_end_modec
    SPI_DELAY	         ; Delay for ~.5us * delay here
no_delay_end_modec:
    ajmp  bang_bits_modec

put_read_val_modec:
    mov   a, R1_B1
    PUTX                         ; Store byte in destination buffer.
    inc   dptr                   ; Advance buffer pointer.
    djnz  r6, modec_no_ss
loop_ss_modec:
    SET_SS                       ; Set SS back to inactive state, to frame data, if required
    jnc   data_loop_modec
    SPI_DELAY	                 ; If using delay, make sure SS stays inactive for sufficent time
    SPI_DELAY	
data_loop_modec:
    djnz  r0, modec_not_done
    djnz  r1, modec_not_done     ; Test for byte loop termination.
    djnz  r2, modec_not_done
    djnz  r3, modec_not_done
    ret
modec_not_done:
    ajmp    SPI_ModeC

;****************************************************************************
;*
;* Function Name: SPI_ModeD
;*
;*   Description: This subroutine handles Mode D SPI.  This corresponds to
;*                CKPOL = 1 and CKPHA = 1
;*
;*      Input(s):
;*
;*    Outputs(s):
;*
;*         Notes:
;*
;****************************************************************************
SPI_ModeD:
    mov   r6, R2_B1              ; Initialize the transfer size counter
    CLEAR_SS                     ; Clear SS line to active state if required
    jnc   moded_no_ss
    SPI_DELAY
    SPI_DELAY			         ; Implement a full clock cycle delay

moded_no_ss:
    GETX                         ; Fetch byte from source buffer.
    mov   R0_B1, a
    mov   r7, #8                 ; Initialize loop counter.
    mov   R1_B1, #0              ; Clear destination register.

bang_bits_moded:
    CLEAR_CLK                    ; Drive CLK low.
    SET_DATA
    jnc   no_delay_moded         ; Test for delay.
    SPI_DELAY		             ; Delay for ~.5us * delay here

no_delay_moded:                  ; Correct for 50% duty here
    mov   R4_B1, #2		 
duty_balance_loop_moded:    
    nop				             ; @14.7Mhz each cycle ~136ns
    nop				             ; 3 nops + 4 for djnz = ~1us
    djnz  R4_B1, duty_balance_loop_moded

after_delay_moded:
    SET_CLK
    GET_DATA
    djnz  r7, data_rotate_d      ; Check for bit loop termination
    nop
    nop
    nop
    nop		; More spi clock balancing and slowing....
    nop		; 10 Cycles to equal other path
    nop
    nop
    nop
    nop
    nop
    jnc   no_delay1_moded
    SPI_DELAY
no_delay1_moded:
    sjmp  put_read_val_moded
data_rotate_d:
    mov   a, R1_B1               ; Rotate destination reg
    rl    a
    mov   R1_B1, a       
    mov   a, R0_B1	             ; Rotate source reg 
    rl    a
    mov   R0_B1, a
    jnc   no_delay_end_moded
    SPI_DELAY	                 ; Delay for ~.5us * delay here
no_delay_end_moded:
    ajmp  bang_bits_moded

put_read_val_moded:
    mov   a, R1_B1
    PUTX                         ; Store byte in destination buffer.
    inc   dptr                   ; Advance buffer pointer.
    djnz  r6, moded_no_ss
loop_ss_moded:
    SET_SS                       ; Set SS back to inactive state, to frame data, if required
    jnc   data_loop_moded
    SPI_DELAY 		             ; If using delay, make sure SS stays inactive for sufficent time
    SPI_DELAY			         ; need to be inactive for 1 clock cycle
data_loop_moded:		         ;    to frame data, if required.	
    djnz  r0, moded_not_done 
    djnz  r1, moded_not_done     ; Test for byte loop termination.
    djnz  r2, moded_not_done
    djnz  r3, moded_not_done
    ret
moded_not_done:
    ajmp  SPI_ModeD

;***************************************************************************
;*
;* Function Name: SPI_AdjustForDJNZ
;*
;*   Description: Adjust loop counter stored in r3-r0 to operate correctly
;*        for use in nested djnz style loop.
;*
;*  Input(s): r3-r0 -> 'regular' loop count
;*
;*    Outputs(s): r3-r0 -> nested djnz loop count
;*
;*     Notes:
;*
;****************************************************************************
spi_AdjustForDJNZ:
    mov   a, r0
    jz    spi_adjust_r1
    inc   r1                 ; If r0 != 0 increment r1

spi_adjust_r1:
    mov   a, r1
    jz    spi_adjust_r2
    inc   r2                 ; If r1 != 0 increment r2

spi_adjust_r2:
    mov   a, r2
    jz    spi_adjust_exit
    inc   r3                 ; If r2 != 0 increment r3

spi_adjust_exit:
    ret

; END of spi_AdjustForDJNZ
;
; ********************************************************************	
	END
